@@ -25,9 +25,9 @@
  *		kolb@yale.edu
  * Date:	Tue Aug 22 1989
  * Copyright (C) 1989 Craig E. Kolb
- * colorquant.c,v 1.1.1.1 1995/08/15 02:46:21 neal Exp
+ * $Id: colorquant.c,v 1.1.1.1 1995/08/15 02:46:21 neal Exp $
  *
- * colorquant.c,v
+ * $Log: colorquant.c,v $
  * Revision 1.1.1.1  1995/08/15 02:46:21  neal
  * initial
  *
@@ -86,7 +86,7 @@
  * "whichbox" in GreatestVariance() is now initialized to 0.
  * 
  */
-static char rcsid[] = "/usr/local/src/CVS/libgr/rle/colorquant.c,v 1.1.1.1 1995/08/15 02:46:21 neal Exp";
+static char rcsid[] = "$Header: /usr/local/src/CVS/libgr/rle/colorquant.c,v 1.1.1.1 1995/08/15 02:46:21 neal Exp $";
 
 #include <math.h>
 #include <stdio.h>
