@@ -1,5 +1,5 @@
 /* minigzip.c -- simulate gzip using the zlib compression library
- * Copyright (C) 1995 Jean-loup Gailly.
+ * Copyright (C) 1995-1996 Jean-loup Gailly.
  * For conditions of distribution and use, see copyright notice in zlib.h 
  */
 
@@ -13,20 +13,19 @@
  * or in pipe mode.
  */
 
-/* $Id: minigzip.c,v 1.1 1995/08/17 01:52:48 neal Exp $ */
+/* $Id: minigzip.c,v 1.3 1996/06/20 02:14:56 neal Exp $ */
 
 #include <stdio.h>
 #include "zlib.h"
 
-#ifndef __GO32__
-extern void exit  __P((int));
-#endif
-extern int unlink __P((const char *));
-
 #ifdef STDC
 #  include <string.h>
+#  include <stdlib.h>
+#else
+   extern void exit  OF((int));
 #endif
 
+
 #if defined(MSDOS) || defined(OS2) || defined(WIN32)
 #  include <fcntl.h>
 #  include <io.h>
@@ -35,6 +34,16 @@ extern int unlink __P((const char *));
 #  define SET_BINARY_MODE(file)
 #endif
 
+#ifdef VMS
+#  define unlink delete
+#  define GZ_SUFFIX "-gz"
+#else
+#  define GZ_SUFFIX ".gz"
+#endif
+#define SUFFIX_LEN sizeof(GZ_SUFFIX)
+
+extern int unlink OF((const char *));
+
 #define BUFLEN 4096
 #define MAX_NAME_LEN 1024
 
@@ -46,18 +55,18 @@ extern int unlink __P((const char *));
 
 char *prog;
 
-void error           __P((char *msg));
-void gz_compress     __P((FILE   *in, gzFile out));
-void gz_uncompress   __P((gzFile in, FILE   *out));
-void file_compress   __P((char  *file));
-void file_uncompress __P((char  *file));
-void main            __P((int argc, char *argv[]));
+void error           OF((const char *msg));
+void gz_compress     OF((FILE   *in, gzFile out));
+void gz_uncompress   OF((gzFile in, FILE   *out));
+void file_compress   OF((char  *file));
+void file_uncompress OF((char  *file));
+int  main            OF((int argc, char *argv[]));
 
 /* ===========================================================================
  * Display error message and exit
  */
 void error(msg)
-    char *msg;
+    const char *msg;
 {
     fprintf(stderr, "%s: %s\n", prog, msg);
     exit(1);
@@ -82,7 +91,7 @@ void gz_compress(in, out)
         }
         if (len == 0) break;
 
-        if (gzwrite(out, buf, len) != len) error(gzerror(out, &err));
+        if (gzwrite(out, buf, (unsigned)len) != len) error(gzerror(out, &err));
     }
     fclose(in);
     if (gzclose(out) != Z_OK) error("failed gzclose");
@@ -104,7 +113,9 @@ void gz_uncompress(in, out)
         if (len < 0) error (gzerror(in, &err));
         if (len == 0) break;
 
-        if (fwrite(buf, 1, len, out) != (uInt)len) error("failed fwrite");
+        if ((int)fwrite(buf, 1, (unsigned)len, out) != len) {
+	    error("failed fwrite");
+	}
     }
     if (fclose(out)) error("failed fclose");
 
@@ -124,14 +135,14 @@ void file_compress(file)
     gzFile out;
 
     strcpy(outfile, file);
-    strcat(outfile, ".gz");
+    strcat(outfile, GZ_SUFFIX);
 
     in = fopen(file, "rb");
     if (in == NULL) {
         perror(file);
         exit(1);
     }
-    out = gzopen(outfile, "wb");
+    out = gzopen(outfile, "wb"); /* use "wb9" for maximal compression */
     if (out == NULL) {
         fprintf(stderr, "%s: can't gzopen %s\n", prog, outfile);
         exit(1);
@@ -156,14 +167,14 @@ void file_uncompress(file)
 
     strcpy(buf, file);
 
-    if (len > 3 && strcmp(file+len-3, ".gz") == 0) {
+    if (len > SUFFIX_LEN && strcmp(file+len-SUFFIX_LEN, GZ_SUFFIX) == 0) {
         infile = file;
         outfile = buf;
         outfile[len-3] = '\0';
     } else {
         outfile = file;
         infile = buf;
-        strcat(infile, ".gz");
+        strcat(infile, GZ_SUFFIX);
     }
     in = gzopen(infile, "rb");
     if (in == NULL) {
@@ -186,7 +197,7 @@ void file_uncompress(file)
  * Usage:  minigzip [-d] [files...]
  */
 
-void main(argc, argv)
+int main(argc, argv)
     int argc;
     char *argv[];
 {
@@ -210,7 +221,7 @@ void main(argc, argv)
             if (file == NULL) error("can't gzdopen stdin");
             gz_uncompress(file, stdout);
         } else {
-            file = gzdopen(fileno(stdout), "wb");
+            file = gzdopen(fileno(stdout), "wb"); /* "wb9" for max compr. */
             if (file == NULL) error("can't gzdopen stdout");
             gz_compress(stdin, file);
         }
@@ -224,4 +235,5 @@ void main(argc, argv)
         } while (argv++, --argc);
     }
     exit(0);
+    return 0; /* to avoid warning */
 }
