@@ -1,9 +1,9 @@
 /* uncompr.c -- decompress a memory buffer
- * Copyright (C) 1995 Jean-loup Gailly.
+ * Copyright (C) 1995-1996 Jean-loup Gailly.
  * For conditions of distribution and use, see copyright notice in zlib.h 
  */
 
-/* $Id: uncompr.c,v 1.1 1995/08/17 01:52:50 neal Exp $ */
+/* $Id: uncompr.c,v 1.3 1996/06/20 02:14:58 neal Exp $ */
 
 #include "zlib.h"
 
@@ -23,15 +23,15 @@
    buffer, or Z_DATA_ERROR if the input data was corrupted.
 */
 int uncompress (dest, destLen, source, sourceLen)
-    Byte *dest;
-    uLong *destLen;
-    Byte *source;
+    Bytef *dest;
+    uLongf *destLen;
+    const Bytef *source;
     uLong sourceLen;
 {
     z_stream stream;
     int err;
 
-    stream.next_in = source;
+    stream.next_in = (Bytef*)source;
     stream.avail_in = (uInt)sourceLen;
     /* Check for source > 64K on 16-bit machine: */
     if ((uLong)stream.avail_in != sourceLen) return Z_BUF_ERROR;
