@@ -1,31 +1,38 @@
 /* zutil.c -- target dependent utility functions for the compression library
- * Copyright (C) 1995 Jean-loup Gailly.
+ * Copyright (C) 1995-1996 Jean-loup Gailly.
  * For conditions of distribution and use, see copyright notice in zlib.h 
  */
 
-/* $Id: zutil.c,v 1.1 1995/08/17 01:52:53 neal Exp $ */
+/* $Id: zutil.c,v 1.3 1996/06/20 02:15:00 neal Exp $ */
 
 #include <stdio.h>
 
 #include "zutil.h"
 
-#ifndef __GO32__
-extern void exit __P((int));
+struct internal_state      {int dummy;}; /* for buggy compilers */
+
+#ifndef STDC
+extern void exit OF((int));
 #endif
 
-char *zlib_version = ZLIB_VERSION;
-
-char *z_errmsg[] = {
-"stream end",          /* Z_STREAM_END    1 */
-"",                    /* Z_OK            0 */
-"file error",          /* Z_ERRNO        (-1) */
-"stream error",        /* Z_STREAM_ERROR (-2) */
-"data error",          /* Z_DATA_ERROR   (-3) */
-"insufficient memory", /* Z_MEM_ERROR    (-4) */
-"buffer error",        /* Z_BUF_ERROR    (-5) */
+const char *z_errmsg[10] = {
+"need dictionary",     /* Z_NEED_DICT       2  */
+"stream end",          /* Z_STREAM_END      1  */
+"",                    /* Z_OK              0  */
+"file error",          /* Z_ERRNO         (-1) */
+"stream error",        /* Z_STREAM_ERROR  (-2) */
+"data error",          /* Z_DATA_ERROR    (-3) */
+"insufficient memory", /* Z_MEM_ERROR     (-4) */
+"buffer error",        /* Z_BUF_ERROR     (-5) */
+"incompatible version",/* Z_VERSION_ERROR (-6) */
 ""};
 
 
+char *zlibVersion()
+{
+    return ZLIB_VERSION;
+}
+
 void z_error (m)
     char *m;
 {
@@ -36,8 +43,8 @@ void z_error (m)
 #ifndef HAVE_MEMCPY
 
 void zmemcpy(dest, source, len)
-    Byte* dest;
-    Byte* source;
+    Bytef* dest;
+    Bytef* source;
     uInt  len;
 {
     if (len == 0) return;
@@ -47,7 +54,7 @@ void zmemcpy(dest, source, len)
 }
 
 void zmemzero(dest, len)
-    Byte* dest;
+    Bytef* dest;
     uInt  len;
 {
     if (len == 0) return;
@@ -57,8 +64,11 @@ void zmemzero(dest, len)
 }
 #endif
 
-#if defined(__TURBOC__) && !defined(__SMALL__)
-
+#ifdef __TURBOC__
+#if (defined( __BORLANDC__) || !defined(SMALL_MEDIUM)) && !defined(__32BIT__)
+/* Small and medium model in Turbo C are for now limited to near allocation
+ * with reduced MAX_WBITS and MAX_MEM_LEVEL
+ */
 #  define MY_ZCALLOC
 
 /* Turbo C malloc() does not allow dynamic allocation of 64K bytes
@@ -73,8 +83,8 @@ void zmemzero(dest, len)
 local int next_ptr = 0;
 
 typedef struct ptr_table_s {
-    voidp org_ptr;
-    voidp new_ptr;
+    voidpf org_ptr;
+    voidpf new_ptr;
 } ptr_table;
 
 local ptr_table table[MAX_PTR];
@@ -85,12 +95,15 @@ local ptr_table table[MAX_PTR];
  * a protected system like OS/2. Use Microsoft C instead.
  */
 
-voidp zcalloc (voidp opaque, unsigned items, unsigned size)
+voidpf zcalloc (voidpf opaque, unsigned items, unsigned size)
 {
-    voidp buf = opaque; /* just to make some compilers happy */
+    voidpf buf = opaque; /* just to make some compilers happy */
     ulg bsize = (ulg)items*size;
 
-    if (bsize < 65536L) {
+    /* If we allocate less than 65520 bytes, we assume that farmalloc
+     * will return a usable pointer which doesn't have to be normalized.
+     */
+    if (bsize < 65520L) {
         buf = farmalloc(bsize);
         if (*(ush*)&buf != 0) return buf;
     } else {
@@ -106,7 +119,7 @@ voidp zcalloc (voidp opaque, unsigned items, unsigned size)
     return buf;
 }
 
-void  zcfree (voidp opaque, voidp ptr)
+void  zcfree (voidpf opaque, voidpf ptr)
 {
     int n;
     if (*(ush*)&ptr != 0) { /* object < 64K */
@@ -127,9 +140,12 @@ void  zcfree (voidp opaque, voidp ptr)
     ptr = opaque; /* just to make some compilers happy */
     z_error("zcfree: ptr not found");
 }
+#endif
 #endif /* __TURBOC__ */
 
-#if defined(M_I86CM) || defined(M_I86LM) /* MSC compact or large model */
+
+#if defined(M_I86) && !(defined(__WATCOMC__) && defined(__386__))
+/* Microsoft C */
 
 #  define MY_ZCALLOC
 
@@ -138,41 +154,43 @@ void  zcfree (voidp opaque, voidp ptr)
 #  define _hfree   hfree
 #endif
 
-voidp zcalloc (voidp opaque, unsigned items, unsigned size)
+voidpf zcalloc (voidpf opaque, unsigned items, unsigned size)
 {
     if (opaque) opaque = 0; /* to make compiler happy */
     return _halloc((long)items, size);
 }
 
-void  zcfree (voidp opaque, voidp ptr)
+void  zcfree (voidpf opaque, voidpf ptr)
 {
     if (opaque) opaque = 0; /* to make compiler happy */
     _hfree(ptr);
 }
 
-#endif /* defined(M_I86CM) || defined(M_I86LM) */
+#endif /* MSC */
 
 
 #ifndef MY_ZCALLOC /* Any system without a special alloc function */
 
-#ifndef __GO32__
-extern voidp calloc __P((uInt items, uInt size));
-extern void  free   __P((voidp ptr));
+#ifndef STDC
+extern voidp  calloc OF((uInt items, uInt size));
+extern void   free   OF((voidpf ptr));
 #endif
 
-voidp zcalloc (opaque, items, size)
-    voidp opaque;
+voidpf zcalloc (opaque, items, size)
+    voidpf opaque;
     unsigned items;
     unsigned size;
 {
-    return calloc(items, size);
+    if (opaque) items += size - size; /* make compiler happy */
+    return (voidpf)calloc(items, size);
 }
 
 void  zcfree (opaque, ptr)
-    voidp opaque;
-    voidp ptr;
+    voidpf opaque;
+    voidpf ptr;
 {
     free(ptr);
+    if (opaque) return; /* make compiler happy */
 }
 
 #endif /* MY_ZCALLOC */
