@@ -1,5 +1,5 @@
 /* zutil.h -- internal interface and configuration of the compression library
- * Copyright (C) 1995 Jean-loup Gailly.
+ * Copyright (C) 1995-1996 Jean-loup Gailly.
  * For conditions of distribution and use, see copyright notice in zlib.h
  */
 
@@ -8,18 +8,14 @@
    subject to change. Applications should only use zlib.h.
  */
 
-/* $Id: zutil.h,v 1.1 1995/08/17 01:52:53 neal Exp $ */
+/* $Id: zutil.h,v 1.3 1996/06/20 02:15:00 neal Exp $ */
 
 #ifndef _Z_UTIL_H
 #define _Z_UTIL_H
 
 #include "zlib.h"
 
-/* #ifndef __GNUC__   disable inline for now */
-#  define inline
-/* #endif */
-
-#ifdef MSDOS
+#if defined(MSDOS) || defined(VMS) || defined(CRAY) || defined(WIN32)
 #   include <stddef.h>
 #   include <errno.h>
 #else
@@ -27,6 +23,7 @@
 #endif
 #ifdef STDC
 #  include <string.h>
+#  include <stdlib.h>
 #endif
 
 #ifndef local
@@ -35,19 +32,25 @@
 /* compile with -Dlocal if your debugger can't find static symbols */
 
 typedef unsigned char  uch;
+typedef uch FAR uchf;
 typedef unsigned short ush;
+typedef ush FAR ushf;
 typedef unsigned long  ulg;
 
-extern char *z_errmsg[]; /* indexed by 1-zlib_error */
+extern const char *z_errmsg[10]; /* indexed by 2-zlib_error */
+/* (size given to avoid silly warnings with Visual C++) */
 
-#define ERR_RETURN(strm,err) return (strm->msg=z_errmsg[1-err], err)
+#define ERR_MSG(err) (char*)z_errmsg[Z_NEED_DICT-(err)]
+
+#define ERR_RETURN(strm,err) \
+  return (strm->msg = ERR_MSG(err), (err))
 /* To be used only when the state is known to be valid */
 
         /* common constants */
 
-#define DEFLATED   8
-
-#define DEF_WBITS 15
+#ifndef DEF_WBITS
+#  define DEF_WBITS MAX_WBITS
+#endif
 /* default windowBits for decompression. MAX_WBITS is for compression only */
 
 #if MAX_MEM_LEVEL >= 8
@@ -66,13 +69,15 @@ extern char *z_errmsg[]; /* indexed by 1-zlib_error */
 #define MAX_MATCH  258
 /* The minimum and maximum match lengths */
 
+#define PRESET_DICT 0x20 /* preset dictionary flag in zlib header */
+
         /* target dependencies */
 
 #ifdef MSDOS
 #  define OS_CODE  0x00
 #  ifdef __TURBOC__
 #    include <alloc.h>
-#  else /* MSC */
+#  else /* MSC or DJGPP */
 #    include <malloc.h>
 #  endif
 #endif
@@ -81,7 +86,7 @@ extern char *z_errmsg[]; /* indexed by 1-zlib_error */
 #  define OS_CODE  0x06
 #endif
 
-#ifdef WIN32 /* Windows NT */
+#ifdef WIN32 /* Window 95 & Windows NT */
 #  define OS_CODE  0x0b
 #endif
 
@@ -109,6 +114,10 @@ extern char *z_errmsg[]; /* indexed by 1-zlib_error */
 
 #ifdef TOPS20
 #  define OS_CODE  0x0a
+#endif
+
+#ifdef _BEOS_
+#  define fdopen(fd,mode) NULL /* No fdopen() */
 #endif
 
         /* Common defaults */
@@ -124,24 +133,37 @@ extern char *z_errmsg[]; /* indexed by 1-zlib_error */
          /* functions */
 
 #ifdef HAVE_STRERROR
-   extern char *strerror __P((int));
+   extern char *strerror OF((int));
 #  define zstrerror(errnum) strerror(errnum)
 #else
 #  define zstrerror(errnum) ""
 #endif
 
-#if defined(pyr) && !defined(NO_MEMCPY)
+#if defined(pyr)
+#  define NO_MEMCPY
+#endif
+#if (defined(M_I86SM) || defined(M_I86MM)) && !defined(_MSC_VER)
+ /* Use our own functions for small and medium model with MSC <= 5.0.
+  * You may have to use the same strategy for Borland C (untested).
+  */
 #  define NO_MEMCPY
 #endif
 #if defined(STDC) && !defined(HAVE_MEMCPY) && !defined(NO_MEMCPY)
 #  define HAVE_MEMCPY
 #endif
 #ifdef HAVE_MEMCPY
-#  define zmemcpy memcpy
-#  define zmemzero(dest, len) memset(dest, 0, len)
+#  ifdef SMALL_MEDIUM /* MSDOS small or medium model */
+#    define zmemcpy _fmemcpy
+#    define zmemcmp _fmemcmp
+#    define zmemzero(dest, len) _fmemset(dest, 0, len)
+#  else
+#    define zmemcpy memcpy
+#    define zmemcmp memcmp
+#    define zmemzero(dest, len) memset(dest, 0, len)
+#  endif
 #else
-   extern void zmemcpy  __P((Byte* dest, Byte* source, uInt len));
-   extern void zmemzero __P((Byte* dest, uInt len));
+   extern void zmemcpy  OF((Bytef* dest, Bytef* source, uInt len));
+   extern void zmemzero OF((Bytef* dest, uInt len));
 #endif
 
 /* Diagnostic functions */
@@ -166,16 +188,16 @@ extern char *z_errmsg[]; /* indexed by 1-zlib_error */
 #endif
 
 
-typedef uLong (*check_func) __P((uLong check, Byte *buf, uInt len));
+typedef uLong (*check_func) OF((uLong check, const Bytef *buf, uInt len));
 
-extern void z_error    __P((char *m));
+extern void z_error    OF((char *m));
 
-voidp zcalloc __P((voidp opaque, unsigned items, unsigned size));
-void  zcfree  __P((voidp opaque, voidp ptr));
+voidpf zcalloc OF((voidpf opaque, unsigned items, unsigned size));
+void   zcfree  OF((voidpf opaque, voidpf ptr));
 
 #define ZALLOC(strm, items, size) \
            (*((strm)->zalloc))((strm)->opaque, (items), (size))
-#define ZFREE(strm, addr)  (*((strm)->zfree))((strm)->opaque, (voidp)(addr))
+#define ZFREE(strm, addr)  (*((strm)->zfree))((strm)->opaque, (voidpf)(addr))
 #define TRY_FREE(s, p) {if (p) ZFREE(s, p);}
 
 #endif /* _Z_UTIL_H */
